<?php

    // use the phpseclib ssh client
    set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__) ."/../../phpseclib");
    require_once('Net/SSH2.php');
    require_once('Crypt/RSA.php');

    class Execute_Remote_SSH_Command_Action extends Reactor_Action
    {
        protected $tpl = 'execute_remote_ssh_command.php';
        protected $name = 'Execute a remote command via SSH';
		
		public static $registration = array(
			ACTION_NAME => 'Remote SSH command',
			ACTION_CATEGORY => 'Network',
			ACTION_DESCRIPTION => 'Logs into a remote machine via SSH and issues a command.',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '01/22/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 
		
        protected $fields = array(
            'hostname' => array(
                'label' => 'remote host',
                'validate' => 'trim|required|xss_clean|callback_check_hostname|prep_for_form',
            ),
            'username' => array(
                'label' => 'remote user',
                'validate' => 'trim|required|alpha_dash|xss_clean|prep_for_form',
            ),
            'auth_type' => array(
                'label' => 'authentication type',
                'validate' => 'trim|required|xss_clean'
            ),
            'password' => array(
                'label' => 'remote password',
                'validate' => 'trim|xss_clean|prep_for_form|callback_check_password',
            ),
            'ssh_key' => array(
                'label' => 'SSH key',
                'validate' => 'trim|xss_clean|prep_for_form|callback_check_ssh_key',
            ),
            'ssh_key_pw' => array(
                'label' => 'SSH key password',
                'validate' => 'trim|xss_clean|callback_check_ssh_key_pw',
            ),
            'command' => array(
                'label' => 'command',
                'validate' => 'trim|required|xss_clean|prep_for_form',
            ),

        );

	
		
        /** 
         * Form validation function to check a hostname.
         **/
        public function check_hostname($str)
        {
            $hostname_expression = '/^(?=.{1,255}$)[0-9A-Za-z]'.
                                   '(?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?'.
                                   '(?:\.[0-9A-Za-z](?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?)*\.?$/i';
            if(!preg_match($hostname_expression, $str)) {
                $this->ci->form_validation->set_message('check_hostname', 'The hostname specified is invalid.');
                return FALSE;
            } else {
                return TRUE;
            }
        }

        /**
         * Form validation function which requires a password to be entered if that auth type is selected.
         **/
        public function check_password($str)
        {
            $type = $this->ci->input->post('auth_type');
            if($type == 'password') {
                if(!strlen($str)) {
                    $this->ci->form_validation->set_message('check_password', 'This field is required for password authentication.');
                    return FALSE;
                }
            }
            return TRUE;
        }
        /**
         * Form validation function which requires an ssh key to be entered if that auth type is selected.
         **/
        public function check_ssh_key($str)
        {
            $type = $this->ci->input->post('auth_type');
            if($type != 'password') {
                if(!strlen($str)) {
                    $this->ci->form_validation->set_message('check_ssh_key', 'This field is required for key authentication.');
                    return FALSE;
                }
            }
            return TRUE;
        }

        /**
         * Form validation function which requires an ssh key password to be entered if that auth type is selected.
         **/
        public function check_ssh_key_pw($str)
        {
            $type = $this->ci->input->post('auth_type');
            if($type == 'key_pw') {
                if(!strlen($str)) {
                    $this->ci->form_validation->set_message('check_ssh_key_pw', 'This field is required.');
                    return FALSE;
                }
            }
            return TRUE;
        }

        /**
         * Get a friendly label for the UI and logs
         **/
		 /*
        public function get_label()
        {
            if($params = $this->get_params()) {
                if(array_key_exists('hostname', $params)) {
                    return sprintf('Remote SSH command on %s', $params['hostname']);
                }
            }
            return parent::get_label();
        }
		*/
         public function get_label()
        {
            if($params = $this->get_params()) {
			
				$command=$params['command'];
				$hostname=$params['hostname'];
               
                return sprintf('Execute command \'%s\' via SSH on host \'%s\'', $command,$hostname);
				}
            return parent::get_label();
        }	
		
        /**
         * Make an SSH connection and run a remote command.
         **/
        public function run($context)
        {
            $this->set_context($context);
            $params = $this->get_params();
            $this->log('info', 'Initializing ssh connection...');
            $ssh = new Net_SSH2($params['hostname']);
            // make this configurable in the ui?
            $ssh->setTimeout(60);
            // authenticate
            switch($params['auth_type']) {
                case "password":
                    $this->log('info', 'Using password authentication.');
                    if(!$ssh->login($params['username'], $params['password'])) {
                        $this->log('error', 'Password authentication failed.');
                        $this->set_result(1);
                        return $context;
                    }
                    break;
                case "key":
                case "key_pw":
                    $key = new Crypt_RSA();
                    if($params['auth_type'] == 'key_pw') {
                        $this->log('info', 'Using password-protected RSA key authentication.');
                        $key->setPassword($params['ssh_key_pw']);
                    } else {
                        $this->log('info', 'Using RSA key authentication.');
                    }
                    $key->loadKey($params['ssh_key']);
                    if(!$ssh->login($params['username'], $key)) {
                        $this->log('error', 'SSH key authentication failed.');
                        $this->set_result(1);
                        return $context;
                    }
                    break;
            }
            $this->log('info', 'Authentication successful. Running remote command: '.$params['command']);
            // run the remote command
            $cmd_out = $ssh->exec($params['command']);
            $this->log('info', 'Got output: '.$cmd_out);
            $this->set_result(0);
            return $context;
        }
    }
